@@ -15,6 +15,7 @@ class ApoJWT():
         server_audience: Audience name of the server hosting the HTTP framework. 
             Audience names are typically base address URLs. Ex: https://example.com
         algorithm: The algorithm to use when encoding/decoding. Default HS256
+        admin_permission: Optional permission allowing full access to JWTs carrying this permission. USE CAREFULLY
         * token_finder: Function used to retrieve the access JWT from http request Authorization header. Default None
 
     Framework Configuration
@@ -26,7 +27,7 @@ class ApoJWT():
             Can be used to format request data in the permission name
     """
 
-    def __init__(self, secret: str, exp_period: int=900, iss: str="", asynch: bool=False, server_audience: str="default", algorithm: str="HS256", token_finder=None, permission_formatter=None, exception_handler=None):        
+    def __init__(self, secret: str, exp_period: int=900, iss: str="", asynch: bool=False, server_audience: str="default", algorithm: str="HS256", admin_permission: str=None, token_finder=None, permission_formatter=None, exception_handler=None):        
         self.exp_period = int(exp_period)
         self.refresh_exp_period = 0
         self.iss = str(iss)
@@ -38,6 +39,7 @@ class ApoJWT():
         self.__exception_handler = exception_handler
         self.__secret = str(secret)
         self.__server_aud = server_audience
+        self.__admin_perm = admin_permission
         self.__refresh_flag = False
         self.__refresh_f = None
         self.__refresh_secret = None
@@ -107,14 +109,17 @@ class ApoJWT():
                     token = self.__token_f(*args, **kwargs)
                     token_dict = self.__decode(token, *args, **kwargs)
                     decoded_permissions = token_dict["perms"]
-                    formatted_perm = permission if self.__permission_formatter is None else self.__permission_formatter(permission=permission, *args, **kwargs)
+                    if self.__admin_perm is not None and self.__admin_perm in decoded_permissions:
+                        return await fn(token_data=token_dict["data"], token_subject=token_dict["sub"], *args, **kwargs)
+
+                    formatted_perm = permission if self.__permission_formatter is None else self.__permission_formatter(permission=permission, *args, **kwargs)                        
                     if formatted_perm not in decoded_permissions:
                         if self.__exception_handler is None:
                             raise jwt.exceptions.InvalidTokenError(f"403: JWT is not authorized for this action")
                         else:
                             self.__exception_handler(code=403, exception_type="unauthorized", msg="JWT is not authorized for this action", *args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
-                    return fn(token_data=token_dict["data"], token_subject=token_dict["sub"], *args, **kwargs)
+                    return await fn(token_data=token_dict["data"], token_subject=token_dict["sub"], *args, **kwargs)
                 return wrapper
             else: 
                 @wraps(fn)
@@ -124,6 +129,9 @@ class ApoJWT():
                     token = self.__token_f(*args, **kwargs)
                     token_dict = self.__decode(token, *args, **kwargs)
                     decoded_permissions = token_dict["perms"]
+                    if self.__admin_perm is not None and self.__admin_perm in decoded_permissions:
+                        return fn(token_data=token_dict["data"], token_subject=token_dict["sub"], *args, **kwargs)
+
                     formatted_perm = permission if self.__permission_formatter is None else self.__permission_formatter(permission, *args, **kwargs)
                     if formatted_perm not in decoded_permissions:
                         if self.__exception_handler is None:
@@ -177,8 +185,7 @@ class ApoJWT():
         permissions: List of permissions to assign to token
         aud: List of audiences token should be accepted by
         data: Any additional information that is needed
-        refresh: Flag to include a refresh token as a second return value
-        refresh_data: If refresh is True, this additional data is stored with the refresh token
+        refresh_data: If refresh is configured, this additional data is stored with the refresh token
 
         JWT will contain the following claims:
             - exp: Expiration Time
